;* Title:		Square Wave Generator
;* Version:		1.0 Beta
;* Target:		AT90S2313
;* CLK frequency:	10.000MHz

.include "2313def.inc"

;***** Constantes

;General:

.equ	MinD	=15		;minimum D, x0.1uS
.equ	MinDm	=1		;minimum D in meander mode, x0.1uS
.equ	MinE	=30		;minimum E, x0.1uS
.equ	DelOff	=1		;delay offset, x0.1uS
.equ	MaxDPE	=9999999	;maximum D,P,E, x0.1uS
.equ	MaxS	=500000		;maximum step, x0.1uS

.equ	MaxW	=50000		;max count in word format
.equ	MinDe	=100		;min delta = OCR1A - TCNT1 for reload
.equ	T0Val	= -98		;x1024x0.0001=10mS program timer
.equ	ARCnV	=16		;slow autorepeat count
.equ	ARDel	=80		;initial autorepeat delay,	x10mS
.equ	ARSlw	=18		;slow autorepeat rate,		x10mS
.equ	ARFst	=6		;fast autorepeat rate,		x10mS
.equ	DebV	=3		;debounce delay,		x10mS

;Keyboard scancodes:

.equ	K_NO	=0x00		;no press
.equ	K_EX	=0x01		;key EXIT code
.equ	K_DN	=0x02		;key DOWN code
.equ	K_UP	=0x04		;key UP code
.equ	K_EN	=0x08		;key ENTER code

;Menu structure:

;[d  0]	<-> [dS 5]
;  |
;[P  1] <-> [PS 6]
;  |
;[SH 2] <-> [EEP 7]
;  |
;[SY 3]
;  |
;[E  4] <-> [ES 8]

.equ	MnuD	=0		;edit Duration
.equ	MnuP	=1		;edit Period
.equ	MnuSH	=2		;edit SHape
.equ	MnuSY	=3		;edit SYnc. mode
.equ	MnuE	=4		;edit dElay
.equ	MnuDS	=5		;edit Duration Step
.equ	MnuPS	=6		;edit Period Step
.equ	MnuST	=7		;STore values in EEPROM
.equ	MnuNo	=8		;not used
.equ	MnuES	=9		;edit dElay Step

.equ	IniMnu	=MnuD		;initial menu item

;ValSH values:

.equ	ShMe	=1		;meander mode
.equ	ShHi	=2		;hi pulse
.equ	ShLo	=3		;lo pulse
.equ	ShFo	=4		;lo level
.equ	ShFi	=5		;hi level

;ValSY values:

.equ	SyAu	=1		;sync. auto
.equ	SyHi	=2		;ext. sync. lo-hi
.equ	SyLo	=3		;ext. sync. hi-lo

;Max menu items:

.equ	MaxSH	=5		;max SH menu item
.equ	MaxSY	=3		;max SY menu item
.equ	MaxST	=1		;max ST menu item

;Periphery control codes:

.equ	CapH	=0x41		;capture L-H, CTC1 = 0 TCCR1B value
.equ	CapL	=0x01		;capture H-L, CTC1 = 0 TCCR1B value
.equ	Comp	=0x09		;compare,     CTC1 = 1 TCCR1B value
.equ	IntDs	=0x00		;int disable           TIMSK  value
.equ	IntCp	=0x08		;capture int enable    TIMSK  value
.equ	IntCm	=0x40		;compare int enable    TIMSK  value
.equ	OutTg	=0x40		;out toggle            TCCR1A value
.equ	OutHi	=0xC0		;out high              TCCR1A value
.equ	OutLo	=0x80		;out low               TCCR1A value
	
;***** Variables

.DSEG	;data segment (internal RAM)

Vals:				;values array	[d,P,SH,SY,E]

ValD:	.byte 3			;duration
ValP:	.byte 3			;period
ValSH:	.byte 3			;shape
ValSY:	.byte 3			;synchronization
ValE:	.byte 3			;delay

Stps:	.byte ValE-ValD+3	;steps array

.equ	Soff	=(Stps-Vals)/3	;steps array index offset

TN0:	.byte 1			;* cycles number (0)
TN1:	.byte 1			;* cycles number (1)

Mem0:	.byte 4			;Memory copy of TH0,TL0,TN0,FLAGS0
Mem1:	.byte 4			;Memory copy of TH1,TL1,TN1,FLAGS1

Buff:	.byte 3			;edit buffer
Step:	.byte 3			;plus/minus step
Max:	.byte 3			;max limit
Min:	.byte 3			;min limit

Dig:	.byte 10		;LCD digits copy

LCode:	.byte 1			;last scancode
TCode:	.byte 1			;temporary scancode

ARCnt:	.byte 1			;autorepeat counter
DebTM:	.byte 1			;debounce timer
KeyTM:	.byte 1			;key timer
MemEnd:				;end of used RAM

.ESEG	;EEPROM initial values

Enone:	.DB 0			;address 0 not used
EVals:				;values array

EValD:	.DB 100,0,0		;duration
EValP:	.DB 200,0,0		;period
EValSH:	.DB 2,0,0		;shape
EValSY:	.DB 1,0,0		;synchronization
EValE:	.DB 100,0,0		;delay

EStps:				;steps array

EStpD:	.DB 1,0,0		;duration step
EStpP:	.DB 1,0,0		;period step
EStpSH:	.DB 1,0,0		;store menu index
EStpSY:	.DB 0,0,0		;not used
EStpE:	.DB 1,0,0		;delay step
Eend:				;end of used EEPROM

;***** Global Register Variables

.def	temp0	=r00		;lpm data register
		
.def	TL0	=r01		;* timer load value L (0)
.def	TH0	=r02		;* timer load value H (0)
.def	TL1	=r03		;* timer load value L (1)
.def	TH1	=r04		;* timer load value H (1)
.def	PCNT	=r05		;* program counter
.def	RgCAP	=r06		;* capture mode restore code

.def	tempQ	=r07		;#
.def	tempP	=r08		;#
.def	XSREG	=r09		;#* SREG save register


.def	tempA	=r13
.def	tempB	=r14
.def	tempC	=r15

.def	tempR	=r16		;#*
.def	temp	=r17		
.def	tempL	=r18		
.def	tempM	=r19		
.def	tempH	=r20		
.def	tempD	=r21
.def	tempE	=r22
.def	tempF	=r23

.def	Mode	=r24
.equ	Loa	=0		;low active level flag
.equ	Fix	=1		;fixed output level flag
.equ	Mea	=2		;meander flag
.equ	Exo	=3		;external sync. fall activate flag
.equ	Exs	=4		;external sync. flag

.def	FLAGS0	=r25		;* flags 0	
.def	FLAGS1	=r26		;* flags 1	
.equ	Sh	=0		;short mode flag
.equ	ShEx	=1		;short ext. mode flag
.equ	Ex	=2		;ext. mode flag
.equ	Lev	=3		;low active level flag

.def	FLAGS2	=r27		;flags 2	
.equ	NEWPR	=0		;new press flag
.equ	UPDD	=1		;display update flag
.equ	EDT	=2		;edited flag
.equ	UPD	=3		;values update flag

.def	Menu	=r29		;menu item

;r28 used as YL register
;r30,r31 used as Z register

;* - used in COMPARE interrupt
;# - used in CAPTURE interrupt

;***** Port Definitions

;DIRx = 1 - out
;PUPx = 1 - IN:pull-up on; OUT: =1

;Port B:

.equ	DIRB	=0b00001111	;directions setup
.equ	PUPB	=0b11110111	;pull-ups and level setup

.equ	DATA	=PB0		;LCD serial data line
.equ	LOAD	=PB1		;LCD load line
.equ	CLK	=PB2		;LCD clock line
.equ	OUT	=PINB3		;generator out
.equ	RETL	=PINB4		;keyboard return line

;Port D:

.equ	DIRD	=0b00100000	;directions setup
.equ	PUPD	=0b11111111	;pull-ups and level setup
.equ	SND	=PD5		;sound out

;***** Macros

.macro	clbr	;clear bit in register
	cbr @0,exp2(@1)
.endm

.macro	stbr	;set bit in register
	sbr @0,exp2(@1)
.endm

.macro	bbrc	;branch if bit in register clear
	sbrs @0,@1
	rjmp @2
.endm

.macro	bbrs	;branch if bit in register set
	sbrc @0,@1
	rjmp @2
.endm

.macro	bbic	;branch if bit in I/O clear
	sbis @0,@1
	rjmp @2
.endm

.macro	bbis	;branch if bit in I/O set
	sbic @0,@1
	rjmp @2
.endm

.macro	ldmi	;load tempH,M,L with 24-bit data
	ldi tempL,low(@0)	
	ldi tempM,high(@0)	
	ldi tempH,byte3(@0)	
.endm

.macro	ldei	;load tempF,E,D with 24-bit data
	ldi tempD,low(@0)	
	ldi tempE,high(@0)	
	ldi tempF,byte3(@0)	
.endm

.macro	IntDIS
;timers interrupts disable
	ldi	temp,IntDs
	out	TIMSK,temp
.endm

.macro	IntEn
;compare interrupt enable for meander mode
	ldi	temp,IntCm
	out	TIFR,temp	;clear pending interrupt
	out	TIMSK,temp	;compare interrupt enable
.endm

.macro	IntCAP
;* capture interrupt enable
	out	TCCR1B,RgCAP	;clear CTC1
	ldi	tempR,IntCp
	out	TIFR,tempR	;clear pending interrupt
	out	TIMSK,tempR	;capture interrupt enable
.endm

.macro	IntCOM
;# compare interrupt enable
	ldi	tempR,Comp
	out	TCCR1B,tempR	;set CTC1
	ldi	tempR,IntCm
	out	TIFR,tempR	;clear pending interrupt
	out	TIMSK,tempR	;compare interrupt enable
.endm

.macro	OutToggle
;#* OC1 toggle	
	ldi	tempR,OutTg
	out	TCCR1A,tempR
.endm	
	
.macro	OutH	
;#* OC1 -> H	
	ldi	tempR,OutHi
	out	TCCR1A,tempR
.endm	
	
.macro	OutL
;#* OC1 -> L	
	ldi	tempR,OutLo
	out	TCCR1A,tempR
.endm

;***** Interrupt Vectors

.CSEG
.org	0			;Reset vector
	rjmp	INIT		
.org	ICP1addr		;Capture interrupt vector
	rjmp	CPTURE		
.org	OC1addr			;Compare interrupt vector

;***** Interrupts Service Routines

;***** Compare interrupt handle (starting at vector):

CMPARE:	sbic	PINB,OUT	;check up out pin
	rjmp	Phase1
	
;Short mode (0):	
	
Phase0:	out	OCR1AH,TH0	;compare register reload (0)
	out	OCR1AL,TL0
	sbrc	FLAGS0,Sh	;check Sh flag
	reti

;Ext. sync. mode (0):

PhEx0:	sbrs	FLAGS0,ShEx	;check ShEx flag
	rjmp	PhLg0
Exdo0:	OutL			;OC1 clear on compare
	IntCAP			;capture interrupt enable
	reti

;Long mode (0):

PhLg0:	in	XSREG,SREG	;save SREG
	tst	PCNT		;test program counter
	brne	Pcnt0
	out	SREG,XSREG	;restore SREG
	sbrc	FLAGS0,Ex	;check Ex flag
	rjmp	Exdo0
	
	lds	PCNT,TN0	;program counter reload
	rjmp	Pcld0
Pcnt0:	dec	PCNT
Pcld0:	tst	PCNT
	breq	Pcze0
	OutL			;OC1 clear on compare
	rjmp	Pcre0
	
Pcze0:	OutToggle		;OC1 toggle on compare
	sbrs	FLAGS0,Ex	;check Ex flag
	rjmp	Pclg0
	
Pcre0:	ldi	tempR,high(MaxW) ;load compare register with const.
	out	OCR1AH,tempR	
	ldi	tempR, low(MaxW)
	out	OCR1AL,tempR
Pclg0:	out	SREG,XSREG	;restore SREG
	reti

;Short mode (1):	
	
Phase1:	out	OCR1AH,TH1	;compare register reload (1)
	out	OCR1AL,TL1
	sbrc	FLAGS1,Sh	;check Sh flag
	reti

;Ext. sync. mode (1):

PhEx1:	sbrs	FLAGS1,ShEx	;check ShEx flag
	rjmp	PhLg1
Exdo1:	OutH			;OC1 set on compare
	IntCAP			;capture interrupt enable
	reti

;Long mode (1):

PhLg1:	in	XSREG,SREG	;save SREG
	tst	PCNT		;test program counter
	brne	Pcnt1
	out	SREG,XSREG	;restore SREG
	sbrc	FLAGS1,Ex	;check Ex flag
	rjmp	Exdo1
	
	lds	PCNT,TN1	;program counter reload
	rjmp	Pcld1
Pcnt1:	dec	PCNT
Pcld1:	tst	PCNT
	breq	Pcze1
	OutH			;OC1 set on compare
	rjmp	Pcre1
	
Pcze1:	OutToggle		;OC1 toggle on compare
	sbrs	FLAGS1,Ex	;check Ex flag
	rjmp	Pclg1

Pcre1:	ldi	tempR,high(MaxW) ;load compare register with const.
	out	OCR1AH,tempR	
	ldi	tempR, low(MaxW)
	out	OCR1AL,tempR
Pclg1:	out	SREG,XSREG	;restore SREG
	reti
	
;***** Capture interrupt handle:	
	
CPTURE:	in	XSREG,SREG	;save SREG
	in	tempQ,ICR1L	;read capture register low  byte
	in	tempP,ICR1H	;read capture register high byte	
	sbrc	FLAGS0,Lev	;check Lev flag (FLAGS0 or FLAGS1)
	rjmp	Cap1
	
;Short delay (0):	
	
Cap0:	add	tempQ,TL0	;add delay
	adc	tempP,TH0
	out	OCR1AH,tempP	;load compare register
	out	OCR1AL,tempQ
	OutToggle		;OC1 toggle on compare
	IntCOM			;compare interrupt enable
	out	SREG,XSREG	;restore SREG
	sbrc	FLAGS0,ShEx	;check ShEx flag
	reti
	
;Long delay (0):	
	
CLg0:	lds	PCNT,TN0	;program counter load
	OutL			;OC1 clear on compare
	out	SREG,XSREG	;restore SREG
	reti
	
;Short delay (1):	

Cap1:	add	tempQ,TL1	;add delay
	adc	tempP,TH1
	out	OCR1AH,tempP	;load compare register
	out	OCR1AL,tempQ
	OutToggle		;OC1 toggle on compare
	IntCOM			;compare interrupt enable
	out	SREG,XSREG	;restore SREG
	sbrc	FLAGS1,ShEx	;check ShEx flag
	reti
	
;Long delay (1):	
	
CLg1:	lds	PCNT,TN1	;program counter load
	OutH			;OC1 set on compare
	out	SREG,XSREG	;restore SREG
	reti
	
;***** Program Execution Starts Here

INIT:	wdr			;Watchdog reset
	ldi	temp,0x0A	;Watchdog enable, 64mS
	out	WDTCR,temp
	
	ldi	temp,RAMEND	;Locate stack
	out	SPL,temp
	
;Ports setup:	
	
	ldi	temp,PUPB
	out	PORTB,temp	;Init PORTB and on/off pullup
	ldi	temp,DIRB	
	out	DDRB,temp	;Set PORTB direction
	
	ldi	temp,PUPD
	out	PORTD,temp	;Init PORTD and on/off pullup
	ldi	temp,DIRD
	out	DDRD,temp	;Set PORTD direction

;Variables init:
	
	ldi	temp,K_NO	;no press code
	sts	LCode,temp
	sts	TCode,temp
	clr	FLAGS2		;clear flags
	
	ldi	YL,Vals
	clr	temp
clear:	st	Y+,temp		;clear all used memory
	cpi	YL,MemEnd
	brne	clear
	
;Periphery setup:

	ldi	temp,0x05
	out	TCCR0,temp	;CK/1024 for TC0
	ldi	temp,Comp
	out	TCCR1B,temp	;CK/1 for TC1
	
;Display init:

	ldi	temp,0x0F	;temp <- BLK register address
	rcall	LCD_WA		;write address
	ldi	temp,0x0F	;temp <- 0x0F - enable bus
	rcall	LCD_WN		;write nibble
	
;Read EEPROM:

	ldi	tempE,EVals	;load initial EEPROM array address
	ldi	YL,Vals		;load initial RAM array address
	
ee_rd:	sbic	EECR,EEWE
	rjmp	ee_rd		;wait for EEPROM ready
	out	EEAR,tempE	;EEPROM address
	sbi	EECR,EERE	;strobe
	in	temp,EEDR	;read EEPROM
	st	Y+,temp		;store byte in RAM
	inc	tempE		;EEPROM address inc
	cpi	tempE,EEnd	;check address
	brne	ee_rd

;Validate all parameters (except D):

	ldi	Menu,MnuES
valall:	rcall	SetMd		;set mode
	rcall	ValidB		;validate buffer and save
	dec	Menu
	brne	valall
	
;Current mode setup:

	ldi	Menu,IniMnu	;display initial menu item
	rcall	ChMd		;setup mode, validate D, SetMd
	
	ldi	tempE,35
	rcall	Tone		;initial beep
	sei			;enable global interrupt
	
;***** Main Loop:

MAIN:	rcall	SCAN		;scan keyboard
	lds	tempL,LCode	
	cp	tempL,temp	;scancode = LCode ?
	breq	Hold		;branch if same key
	
	lds	tempL,TCode
	cp	tempL,temp	;csancode = TCode ?
	brne	NewP		;branch if new key
	
	lds	tempL,DebTM
	tst	tempL		;check debounce timer
	brne	Hold
	
	sts	LCode,temp	;LCode <- scancode
	stbr	FLAGS2,NEWPR	;set new press flag
	ldi	tempL,ARDel
	sts	KeyTM,tempL	;load KeyTM with autorepeat delay
	ldi	tempL,ARCnV
	sts	ARCnt,tempL	;load autorepeat counter
	rjmp	Proc		;jump to process
	
NewP:	sts	TCode,temp	;TCode <- scancode
	ldi	tempL,DebV
	sts	DebTM,tempL	;debounce timer load
	rjmp	Hold
	
Hold:	clbr	FLAGS2,NEWPR	;clear new press flag
	lds	tempL,KeyTM
	tst	tempL		;check key timer
	brne	Proc
	stbr	FLAGS2,NEWPR	;set new press flag
	lds	tempL,ARCnt
	tst	tempL
	breq	Fast		;fast autorepeat if count is over
	dec	tempL		;dec autorepeat counter
	sts	ARCnt,tempL
	ldi	tempL,ARSlw
	sts	KeyTM,tempL	;load slow autorepeat rate
	rjmp	Proc
Fast:	ldi	tempL,ARFst
	sts	KeyTM,tempL	;load fast autorepeat rate
Proc:
	
;LCode	= true scancode,
;NEWPR	= 1 if new press

;Process key functions:
	
	bbrc	FLAGS2,NEWPR,NoPr
	lds	temp,LCode
	cpi	temp,K_NO
	breq	NoPr		;skip if no press
	rcall	Sound		;key beep
	cpi	temp,K_UP
	brne	Pro1
	rcall	Do_UP		;key UP processing
Pro1:	cpi	temp,K_DN
	brne	Pro2
	rcall	Do_DN		;key DN processing
Pro2:	cpi	temp,K_EX
	brne	Pro3
	rcall	Do_EX		;key EX processing
Pro3:	cpi	temp,K_EN
	brne	NoPr
	rcall	Do_EN		;key EN procesing
NoPr:

;Process timers:	
	
	in	temp,TIFR
	bbrc	temp,TOV0,Novf	;check for Timer 0 overflow
	stbr	temp,TOV0	;Timer 0 overflow flag clear
	out	TIFR,temp
		
	ldi	temp,T0Val
	out	TCNT0,temp	;Timer 0 reload
	
	lds	temp,DebTM
	tst	temp		;check debounce timer		 
	breq	Tmr1
	dec	temp		;advance debounce timer
	sts	DebTM,temp

Tmr1:	lds	temp,KeyTM
	tst	temp		;check key timer		 
	breq	Tmr2
	dec	temp		;advance key timer
	sts	KeyTM,temp
Tmr2:	
Novf:

;Update display:

	bbrc	FLAGS2,UPDD,NoUpd ;check up update flag
	clbr	FLAGS2,UPDD	  ;clear update flag
	rcall	Disp		  ;update display
NoUpd:

;Safe update values: 

	bbrc	FLAGS2,UPD,Skip	;check up values update flag
	lds	tempL,TN0	;tempL <- TN0
	lds	tempH,TN1	;tempH <- TN1
	or	tempL,tempH
	breq	Short		;TN0 and TN1 = 0
	
	tst	PCNT
	brne	Skip		;wait for PCNT = 0
	rcall	Delta		;delta = OCR1A - TCNT1
	brcs	Skip		;skif if delta < MinDe
	
Short:	bbrc	Mode,Mea,DoLoad	;not meander mode, immediate load
	lds	temp,Mem0+2	;load new TN0
	tst	temp		;check mode (short or long)		
	breq	smmd		
lmmd:	IntEn			;long meander mode, int. enable
	rjmp	DoLoad
smmd:	IntDis			;short meander mode, int disable
	lds	tempH,Mem0+0
	lds	tempL,Mem0+1
	out	OCR1AH,tempH	;load capture register in short mode
	out	OCR1AL,tempL

DoLoad:	rcall	ImLoad		;copy data from memory buffer
	clbr	FLAGS2,UPD	;all updated, clear flag
Skip:	

;Save values:

	cpi	Menu,MnuST	;check for STore menu
	brne	NoSt	

;Write to EEPROM:
	
	ldi	tempE,EVals	;load initial EEPROM array address
	ldi	YL,Vals		;load initial RAM array address
	
ee_wr:	sbic	EECR,EEWE
	rjmp	ee_wr		;wait for EEPROM ready
	ld	temp,Y+		;read RAM
	out	EEAR,tempE	;EEPROM address
	sbi	EECR,EERE	;strobe
	in	tempD,EEDR	;read EEPROM
	cp	tempD,temp
	breq	nx_wr		;skip write if same
	out	EEDR,temp	;load data
	cli			;interrupts disable
	sbi 	EECR,EEMWE	;master write enable
	sbi	EECR,EEWE	;strobe
	sei			;interrupts enable
nx_wr:	wdr			;watchdog restart
	inc	tempE		;EEPROM address inc
	cpi	tempE,EEnd	;check address
	brne	ee_wr

	ldi	Menu,MnuSH
	rcall	SetMd		;set SHape menu
	ldi	tempE,50	;data save melody
	rcall	Tone
	ldi	tempE,45
	rcall	Tone
	ldi	tempE,35
	rcall	Tone
NoSt:

;Watchdog restart:

	wdr			;watchdog restart
	rjmp	MAIN
	
;***** Subroutines

;Key "UP" processing:

Do_UP:	cpi	Menu,MnuDS	;step edit ?
	brlo	upns
	
;Step change (1 -> 2 -> 5 -> 10 -> 20..)

	rcall	LdBuff		;tempH,M,L <- Buff
	lsl	tempL
	rol	tempM
	rol	tempH		;Buff * 2
	rcall	Search
	cpi	temp,2
	brne	vbuf		;validate buffer
	ldi	YL,Buff
	rcall	LdABC		;tempC,B,A <- Buff	
	lsr	tempC
	ror	tempB
	ror	tempA		;Buff / 2
	add	tempL,tempA
	adc	tempM,tempB
	adc	tempH,tempC	;Buff * 2.5
	rjmp	vbuf		;validate buffer

;Buff + Step, align value to step

upns:	rcall	Rem		;tempH,M,L <- Buff, tempF,E,D <- Step, tempC,B,A <- Rem
	sub	tempL,tempA	;Buff - Rem(Buff/Step)
	sbc	tempM,tempB
	sbc	tempH,tempC
	add	tempL,tempD	;Buff + Step
	adc	tempM,tempE
	adc	tempH,tempF
	rjmp	vbuf		;validate buffer
	
;Key "DN" processing:
		
Do_DN:	cpi	Menu,MnuDS	;step edit ?
	brlo	dnns
	
;Step change (..20 -> 10 -> 5 -> 2 -> 1)

	rcall	LdBuff		;load Buff to tempH,M,L
	lsr	tempH
	ror	tempM
	ror	tempL		;Buff / 2
	rcall	Search
	cpi	temp,5
	brne	vbuf		;validate buffer
	rcall	LdBuff		;load Buff to tempH,M,L	
	lsl	tempL
	rol	tempM
	rol	tempH		;Buff * 2
	ldi	tempD,5
	clr	tempE
	clr	tempF
	rcall	Div24
	com	tempL
	com	tempM
	com	tempH		;Buff / 2.5
	rjmp	vbuf		;validate buffer

;Buff - Step, align value to step

dnns:	rcall	Rem		;tempH,M,L <- Buff, tempF,E,D <- Step, tempC,B,A <- Rem
	mov	temp,tempA
	or	temp,tempB
	or	temp,tempC
	breq	dn0
	sub	tempL,tempA	;Buff - Rem(Buff/Step)
	sbc	tempM,tempB
	sbc	tempH,tempC
	rjmp	dn1		;validate buffer	
dn0:	sub	tempL,tempD	;Buff - Step
	sbc	tempM,tempE
	sbc	tempH,tempF
dn1:
	
;Auto change step
	
	cpi	tempD,1		;tempD = 1 means Step = 1
	breq	vbuf		;Step = 1, nothing to change
	rcall	ChkMin		;check for Min
	brcc	vbuf		;no overflow, validate buffer
	push	Menu		;save curent menu item
	subi	Menu,-Soff	;change step mode
	rcall	SetMd		;set change step mode
	rcall	Disp		;fill [Dig...]
	rcall	ErrB		;error bell
	rcall	Do_DN		;step down
	pop	Menu
	rcall	SetMd		;restore mode
	ret

vbuf:	rcall	Valid		;validate tempH,M,L
	rcall	Assume		;Assume new parameters
	stbr	FLAGS2,UPDD	;set display update flag
	stbr	FLAGS2,EDT	;set edited flag
	ldi	temp,K_NO	;end of processing
	ret
	
;Load tempH,M,L from Buff

LdBuff:	ldi	YL,Buff

;Load tempH,M,L from [Y+2],[Y+1],[Y+0]

LdLMN:	ld	tempL,Y+
	ld	tempM,Y+
	ld	tempH,Y+
	ret
	
;Load tempC,B,A from [Y+2],[Y+1],[Y+0]

LdABC:	ld	tempA,Y+
	ld	tempB,Y+
	ld	tempC,Y+
	ret
	
;Load tempF,E,D from [Y+2],[Y+1],[Y+0]

LdDEF:	ld	tempD,Y+
	ld	tempE,Y+
	ld	tempF,Y+
	ret
		
;tempH,L,M = Buff, tempF,E,D = Step, tempC,B,A = rem(Buff/Step)

Rem:	rcall	LdBuff		;load Buff to tempH,M,L
	ldi	YL,Step
	rcall	LdDEF
	rcall	div24
done:	rcall	LdBuff		;load Buff to tempH,M,L
	ret
	
;Divide tempH,M,L / tempF,E,D = ~tempH,M,L , tempC,B,A

div24:	clr	tempA		;clear remainder Low byte
	clr	tempB
	sub	tempC,tempC	;clear remainder High byte and carry
	
	ldi	temp,25		;init loop counter
	
div1:	rol	tempL		;shift left dividend
	rol	tempM
	rol	tempH
	dec	temp		;decrement counter
	brne	div2
	ret
div2:	rol	tempA		;shift dividend into remainder
	rol	tempB
	rol	tempC
	sub	tempA,tempD	;remainder = remainder - divisor
	sbc	tempB,tempE	
	sbc	tempC,tempF	
	brcc	div1		;if result negative
	add	tempA,tempD	; restore remainder
	adc	tempB,tempE
	adc	tempC,tempF
	rjmp	div1

;Search for first non-zero digit:

Search:	ldi	YL,Dig+3
srch:	ld	temp,Y+
	andi	temp,~Pt
	breq	srch
	cpi	temp,BLANK
	breq	srch
	ret
	
;Validate:
;Input: [Buff], [Max], [Min] - limits
;Out:   [Buff], [Vals[Menu]]

ValidB:	rcall	LdBuff		;tempH,M,L <- [Buff]

;Validate:
;Input: tempH,M,L, [Max], [Min] - limits
;Out:   [Buff], [Vals[Menu]]

Valid:	rcall	ChkMin		;check for Min
	brcs	RstBf
	rcall	ChkMax		;check for Max
	brcs	RstBf
	rjmp	ValOk	
	
RstBf:	rcall	LdBuff		;restore buffer	
	rcall	ErrB		;error bell
	rcall	ChkMin		;check for Min
	brcs	LimBf
	rcall	ChkMax		;check for Max
	brcs	LimBf
	rjmp	ValOk	
	
LimBf:	mov	tempL,tempA	;limit buffer
	mov	tempM,tempB
	mov	tempH,tempC
	rcall	ErrB		;error bell

ValOk:	ldi	YL,Buff
	rcall	StY		;save buffer
	ldi	YL,Vals
	add	YL,Menu
	add	YL,Menu
	add	YL,Menu		;YL = Vals + Menu * 3
	rcall	StY		;Save Buff (tempH,M,L) to [Vals+temp*3]
	ret
	
;Compare tempH,M,L and Min:
;Out: C = 1 if limit exceeded
	
ChkMin:	ldi	YL,Min
	rcall	LdABC		;tempC,B,A <- Min
	cpi	tempH,0xC0
	brcs	chm		;0xC00000 is max negative
	sei
	rjmp	chmr
chm:	cp	tempL,tempA
	cpc	tempM,tempB
	cpc	tempH,tempC
chmr:	ret

;Compare tempH,M,L and Max:
;Out: C = 1 if limit exceeded

ChkMax:	ldi	YL,Max
	rcall	LdABC		;tempC,B,A <- Max
	cp	tempA,tempL
	cpc	tempB,tempM
	cpc	tempC,tempH
	ret

;Change mode (SH or SY):

;Mode format:
;
;ValSH:       ShMe  ShHi  ShLo  ShFo  ShFi
;
;Mode.0	(Loa)  0     0     1     0     1
;Mode.1	(Fix)  0     0     0     1     1
;Mode.2	(Mea)  1     0     0     0     0
;
;ValSY:       SyAu  SyHi  SyLo
;
;Mode.3	(Exo)  1     0     1
;Mode.4	(Exs)  0     1     1

ChMd:	IntDis			;interrupts disable
	clr	PCNT		;clear program counter
	
;Compose Mode variable
	
	ldi	YL,ValSH
	rcall	ldLMN		;tempL <- ValSH
	subi	tempL,2
	brcc	nme
	andi	tempL,4
nme:	mov	Mode,tempL	;setup Loa, Fix, Mea
	ldi	YL,ValSY
	rcall	ldLMN		;tempL <- ValSY
	swap	tempL
	lsr	tempL
	or	Mode,tempL	;setup Exo, Exs
	
;Validate D	
	
	push	Menu
	ldi	Menu,MnuD
	rcall	SetMd		;Buff <- D
	rcall	ValidB		;validate D, save D
	pop	Menu
	rcall	SetMd		;restore mode
	
;Setup output level
	
	bbrc	Mode,Loa,hoa	;check up level flag
	OutH			;OC1 high
	rjmp	fxp
hoa:	OutL			;OC1 low
fxp:	sbrc	Mode,Fix
	ret			;return if fixed output level
	bbrs	Mode,Mea,syna	;meander mode, auto sync.
	bbrc	Mode,Exs,syna	;Exs=0, auto sync.
	
;External sync.	
	
	bbrc	Mode,Exo,exi	;check ext. sync. polarity
	ldi	temp,CapL	;capture H-L
	mov	RgCAP,temp	;save capture setup value
	rjmp	syne
exi:	ldi	temp,CapH	;capture L-H
	mov	RgCAP,temp	;save capture setup value
syne:	rcall	LoadD		;load D to memory buffer
	rcall	LoadE		;load E to memory buffer
	rcall	ImLoad		;immediate load
	IntCAP			;capture interrupt enable
	ret

;Assume new parameters:
;Input: Menu - index
	
Assume:	cpi	Menu,MnuSH
	breq	ChMd		;change mode
	cpi	Menu,MnuSY
	breq	ChMd		;change mode
	cpi	Menu,MnuD
	breq	LoadD		;change D
	cpi	Menu,MnuP
	breq	LoadP		;change P
	cpi	Menu,MnuE
	breq	LoadE		;change E
	ret

;Auto sync.

syna:	rcall	LoadD		;load D to memory buffer
	rcall	LoadP		;load P to memory buffer
	rcall	ImLoad		;immediate load
	OutToggle		;OC1 toggle
	sbrc	Mode,Mea	;meander mode, int. disable
	ret
	IntCOM			;compare interrupt enable
	ret

;Load duration to memory buffer:

LoadD:	ldi	YL,ValD
	rcall	Split		;tempC,B,A <- TN,TH,TL, tempD <- flags set
	
LdAct:	ldi	YL,Mem0
	sbrs	Mode,Loa	;check up active level
	ldi	YL,Mem1		;YL- pointer to Mem0 or Mem1
	st	Y+,tempB	;fill memory buffer
	st	Y+,tempA
	st	Y+,tempC
	st	Y+,tempD
	
;Load period to memory buffer:

LoadP:	ldi	YL,ValD
	bbrs	Mode,Mea,pme	;check up meander mode
	bbrs	Mode,Exs,Nmm	;check up external sync. mode
	rcall	LdABC		;tempC,B,A <- ValD
	ldi	YL,ValP
	rcall	LdLMN		;tempH,M,L <- ValP
	sub	tempL,tempA	;calculate inactive level duration
	sbc	tempM,tempB
	sbc	tempH,tempC
	rcall	SplitH		;tempC,B,A <- TN,TH,TL, tempD <- flags set
	rjmp	LdPas
pme:	rcall	Split	
	
LdPas:	ldi	YL,Mem0		
	sbrc	Mode,Loa	;check up active level
	ldi	YL,Mem1		;YL- pointer to Mem0 or Mem1
	st	Y+,tempB	;fill memory buffer
	st	Y+,tempA
	st	Y+,tempC
	st	Y+,tempD
Nmm:	stbr	FLAGS2,UPD	;update request
	ret
	
;Load delay to memory buffer:	
	
LoadE:	ldi	YL,ValE
	rcall	LdLMN		;tempH,M,L <- ValE
	subi	tempL,DelOff	;ValE - Delay offset
	sbci	tempM,0
	sbci	tempH,0
	rcall	SplitH		;tempC,B,A <- TN,TH,TL, tempD <- flags set
	sbrc	tempD,Sh
	stbr	tempD,ShEx	;ShEx <- Sh
	clbr	tempD,Sh
	stbr	tempD,Ex	;set Ex flag
	rjmp	LdPas

;Splits 24-bit value:
;Input: YL - pointer to value,

Split:	rcall	LdLMN		;load tempH,M,L from [Y]

;Splits 24-bit value:
;Input: tempH,M,L - value,
;       N - counter value (byte),
;       T - timer value (word)
;N.R = value / MaxW, if N=0 - short mode
;if R<(0xffff-MaxW) then R=R+MaxW, N=N-1
;R=R-1, if N=0 - short mode
;Out: TL = R(L) - tempA, TH = R(H) - tempB, TN = N    - tempC
;Input: Mode, Out:   tempD = flags set

SplitH:	ldei	MaxW		;tempF,E,D <- MaxW
	rcall	Div24		;~tempH,M,L . tempC,B,A = value / MaxW
	com	tempL
	breq	shm		;N=0
	ldi	tempH,high(0xffff-MaxW)
	ldi	tempM,low (0xffff-MaxW)
	cp	tempA,tempM
	cpc	tempB,tempH
	brsh	shm		;R>=0xffff-MaxW
	add	tempA,tempD	;R=R+MaxW
	adc	tempB,tempE
	dec	tempL		;N=N-1
shm:	ldi	tempM,1
	ldi	tempH,0	
	sub	tempA,tempM	;R=R-1
	sbc	tempB,tempH
adone:	mov	tempC,tempL

;Flags setup

	clr	tempD		;flags set init
	sbrc	Mode,Loa
	stbr	tempD,Lev	;Lev <- Loa
	tst	tempC		;check N for zero
	brne	lgmd		;N > 0, long mode
	stbr	tempD,Sh	;short mode, set Sh
lgmd:	ret
	
;Compare delta = OCR1A - TCNT1 and MinDe
;Out: C = 1 if delta < MinDe

Delta:	in	tempM,TCNT1L
	in	tempH,TCNT1H
	in	tempD,OCR1AL
	in	tempE,OCR1AH
	sub	tempD,tempM	;delta = OCR1A - TCNT1
	sbc	tempE,tempH
	subi	tempD,low (MinDe)
	sbci	tempE,high(MinDe)
	ret
	
;Immediate load interrupts parameters from memory buffer:

ImLoad:	ldi	YL,Mem0
	ld	TH0,Y+		;load TH0
	ld	TL0,Y+		;load TL0
	ld	temp,Y+
	sts	TN0,temp	;load TN0
	ld	FLAGS0,Y+	;load FLAGS0
	
	ld	TH1,Y+		;load TH1
	ld	TL1,Y+		;load TL1
	ld	temp,Y+
	sts	TN1,temp	;load TN1
	ld	FLAGS1,Y+	;load FLAGS1
	ret

;Key "EX" processing (change menu item):
	
Do_EX:	cpi	Menu,MnuDS
	brsh	Do_EN		;enter if steps
	mov	temp,Menu
	inc	temp
		
	cpi	Menu,MnuD
	brne	ex1
	sbrc	Mode,Mea
	ldi	temp,MnuSH	;Mea = 1, MnuD -> MnuSH
	bbrc	Mode,Exs,ex1
	ldi	temp,MnuSH	;Exs = 1, MnuD -> MnuSH
	sbrc	FLAGS2,EDT
	ldi	temp,MnuE	;EDT = 1, MnuD -> MnuE
	
ex1:	cpi	Menu,MnuP
	brne	ex2
	sbrc	FLAGS2,EDT
	ldi	temp,MnuD	;EDT = 1, MnuP -> MnuD
	
ex2:	cpi	Menu,MnuSH
	brne	ex3
	sbrc	Mode,Mea	
	ldi	temp,MnuD	;Mea = 1, MnuSH -> MnuD
	
ex3:	cpi	Menu,MnuSY
	brne	ex4
	sbrs	Mode,Exs	
	ldi	temp,MnuD	;Exs = 0, MnuSY -> MnuD
	
ex4:	cpi	Menu,MnuE
	brne	ex5
	ldi	temp,MnuD	;MnuE -> MnuD

ex5:	mov	Menu,temp
ex6:	rcall	SetMd
	clbr	FLAGS2,EDT	;clear edited flag
	ldi	temp,K_NO	;end of processing
	ret	
	
;Key "EN" processing (change menu item):

Do_EN:	cpi	Menu,MnuSY
	breq	en3		;error if Mnu=MnuSY
	cpi	Menu,MnuDS
	brlo	en1
	subi	Menu,Soff
	ldi	temp,0x00
	rcall	Sound		;return beep
	rjmp	ex6
en1:	subi	Menu,-Soff
	rjmp	ex6
en3:	rcall	ErrB
	ldi	temp,K_NO	;end of processing
	ret
	
;Set current edit mode:

SetMd:	lsl	Menu		;*2
	ldi	ZL,low (StrT*2)	;string table base
	ldi	ZH,high(StrT*2)
	ldi	YL,Dig+0
	rcall	RdTbl		;read char 1
	st	Y+,temp
	rcall	RdTblN		;read char 2
	st	Y+,temp
	lsr	Menu		;/2
	
;Load Buff from [Vals[Menu * 3]]
	
	ldi	YL,Vals
	rcall	LdLMNM		;tempH,M,L <- [Y + Menu * 3]
	ldi	YL,Buff
	rcall	StY		;[Buff] <- tempH,M,L
	
;Check for special menu cases	
	
	cpi	Menu,MnuST
	breq	mst	
	cpi	Menu,MnuSH
	breq	msh
	cpi	Menu,MnuSY
	breq	msy
	cpi	Menu,MnuDS
	brsh	stp		;Menu>=MnuDS - steps
	
;Load Step from	[Stps[Menu * 3]]
	
	ldi	YL,Stps
	rcall	LdLMNM		;tempH,M,L <- [Y + Menu * 3]
	ldi	YL,Step
	rcall	StY		;[Step] <- tempH,M,L
	
;Load Min and Max	
	
	cpi	Menu,MnuE
	breq	mne
	cpi	Menu,MnuP
	breq	mnp
mnd:	bbrc	Mode,Mea,mnm
	ldmi	MinDm		;Min for D = MinDm in meander mode
	rjmp	sma	
mnm:	ldmi	MinD		;Min for D = MinD
	bbrs	Mode,Exs,sma
	ldi	YL,ValP		;Max for D = ValP - MinD
	rcall	LdDEF		;tempF,E,D <- ValP
	subi	tempD,low(MinD)
	sbci	tempE,high(MinD)
	sbci	tempF,byte3(MinD)
	rjmp	stl
mnp:	ldi	YL,ValD		;Min for P = ValD + MinD
	rcall	LdLMN		;tempH,M,L <- ValD
	subi	tempL,low(-MinD)
	sbci	tempM,high(-MinD)
	sbci	tempH,byte3(-MinD)
	rjmp	sma	
mne:	ldmi	MinE		;Min for E
sma:	ldei	MaxDPE		;Max for E, P and D if meander or ext.	
	rjmp	stl

;Load step limits

stp:	ldei	MaxS		;Max for steps
	rjmp	msm

;Special menu cases

msy:	ldei	MaxSY		;Max for SY menu			
	rjmp	msm
mst:	ldei	MaxST		;Max for EEP menu
	rjmp	msm
msh:	ldei	MaxSH		;Max for SH menu
			
msm:	ldmi	1		;Step and Min	
	ldi	YL,Step
	rcall	StY		;store Step
	
;Load Min and Max from tempH,M,L and tempF,E,D
	
stl:	rcall	StMin		;store Min from tempH,M,L
	rcall	StMax		;store Max from tempF,E,D
	stbr	FLAGS2,UPDD
	ret

;Read table to temp:
;Z - base, Menu - offset

RdTbl:	add	ZL,Menu		;add offset (Z + Menu)
	brcc	rdt1
	inc	ZH
rdt1:	lpm			;temp0 <- item
	mov	temp,temp0
	ret

;Read next byte from table to temp:
;Z - pointer

RdTblN:	adiw	ZL,1		;advance pointer
	lpm			;read next byte
	mov	temp,temp0
	ret
	
;Store min limit from tempH,M,L	
	
StMin:	ldi	YL,Min

;Store 3  bytes in memory at [Y+2],[Y+1],[Y+0]
;from tempH,M,L

StY:	st	Y+,tempL
	st	Y+,tempM
	st	Y+,tempH
	ret
	
;Store max limit from tempF,E,D	
	
StMax:	ldi	YL,Max

;Store 3  bytes in memory at [Y+2],[Y+1],[Y+0]
;from tempH,M,L

StYe:	st	Y+,tempD
	st	Y+,tempE
	st	Y+,tempF
	ret
	
;Load tempH,M,L from [Y + Menu * 3]	
	
LdLMNM:	add	YL,Menu
	add	YL,Menu
	add	YL,Menu		;YL = YL + Menu * 3
	rjmp	LdLMN
	
;Display current editing value:
;Dig[3..9] <- Buff (convert to BCD) or
;Dig[3..9] <- ShpT, SynT string
;Supress leading zeros
;Add point
;LCD <- Dig[0..9]

Disp:	ldi	YL,Dig+2
	ldi	temp,BLANK
blout:	st	Y+,temp
	cpi	YL,Dig+10
	brne	blout		;blank display
	
	rcall	LdBuff		;load Buff to tempH,M,L
	
;Dig[3..8] <- strings
	
	cpi	Menu,MnuSH
	brne	notsh
	ldi	ZL,low (ShpT*2)	;shape string table base
	ldi	ZH,high(ShpT*2)
	rjmp	cops

notsh:	cpi	Menu,MnuSY
	brne	notsy
	ldi	ZL,low (SynT*2)	;sync string table base
	ldi	ZH,high(SynT*2)
	rjmp	cops
	
notsy:	cpi	Menu,MnuST
	brne	notst
	ldi	ZL,low (SttT*2)	;store string table base
	ldi	ZH,high(SttT*2)
	
;Copy string (5 chars) from table to Dig[4..7]
;Z - table base, tempL - string number

cops:	push	Menu
	dec	tempL
	lsl	tempL		;*2
	lsl	tempL		;*4
	mov	Menu,tempL
	rcall	RdTbl		;read table (byte 1)
	ldi	YL,Dig+4
cpyst:	st	Y+,temp
	rcall	RdTblN		;read next byte
	cpi	YL,Dig+8
	brne	cpyst
	pop	Menu
	rjmp	dload
	
;Buff[0..3] convert to BCD Dig[3..9]

notst:	ldi	YL,Dig+3
	clr	temp
clrout:	st	Y+,temp
	cpi	YL,Dig+10
	brne	clrout		;output array clear
	
	ldi	tempD,24	;input bits count
	ldi	ZL,Dig+3
	
hloop:	lsl	tempL		;input array shift left
	rol	tempM
	rol	tempH		
	ldi	YL,Dig+10
sloop:	ld	temp,-Y
	rol	temp
	subi	temp,-0x06	;temp+6, C=1
	sbrs	temp,4
	subi	temp,0x06	;temp-6, C=0
	andi	temp,0x0f
	st	Y,temp
	cpse	YL,ZL
	rjmp	sloop
	dec	tempD
	brne	hloop
	
;Zeros suppresion	
	
	ldi	YL,Dig+3
	ldi	tempL,BLANK
zsupp:	ld	temp,Y
	tst	temp
	brne	notz
	st	Y+,tempL	;blank zero
	cpi	YL,Dig+8
	brne	zsupp
	
;Setup points	
	
notz:	ldi	YL,Dig+5
	ld	temp,Y
	cpi	temp,BLANK
	breq	blnk5
	ori	temp,Pt
	st	Y,temp		;point at Dig+5
	rjmp	dload
	
blnk5:	ldi	YL,Dig+8
	ld	temp,Y
	ori	temp,Pt
	st	Y,temp		;point at Dig+8
	
;Indicate Dig[0..9]
	
dload:	ldi	temp,0x00	;temp <- 0x00 - SG1 address
	rcall	LCD_WA		;write address
	ldi	YL,Dig
ld1:	ld	temp,Y+		;temp <- digit

;Write symbol from temp to LCD:
;temp.7 - point

.equ	Pt	=0x80

	bst	temp,7		;T <- temp.7
	andi	temp,0x7F	;temp.7 <- 0
	ldi	ZL,low (FONT*2)	;char table base
	ldi	ZH,high(FONT*2)
	add	ZL,temp		;add offset (Z + temp)
	brcc	wc1
	inc	ZH
wc1:	lpm			;temp0 <- char
	mov	temp,temp0	;temp  <- char
	bld	temp,H
	
;Write data from temp to LCD:

	push	temp
	rcall	LCD_WN		;Write nibble from temp to LCD
	pop	temp
	swap	temp
	rcall	LCD_WN		;Write nibble from temp to LCD
	
	cpi	YL,Dig+10
	brne	ld1		;repeat for 10 digits
	ret	

;***** Beeper support:

;Error bell:

ErrB:	ldi	tempE,55	;period for error bell

;Sound generation:
;Input: tempE - period

Tone:	ldi	tempF,255	;tone duration
	rjmp	Sgen	

;Sound generation:
;Input: temp - frequency

Sound:	mov	tempE,temp	;temp - frequency
	com	tempE		;temp=~temp
	andi	tempE,0x0F	;mask unused bits
	subi	tempE,-30	;add period offset
	ldi	tempF,40	;sound duration
	
;Sound generation:
;Input: tempL - period
;       tempE - duration
	
Sgen:	mov	tempA,tempE	;3
sndb:	ldi	tempD,40	  ;2 outer loop
snda:	dec	tempD		    ;1 inner loop
	brne	snda		    ;1 inner loop
	dec	tempA		  ;2 outer loop
	brne	sndb		  ;2 outer loop
	dec	tempF		;3
	wdr			;3 watchdog restart
	sbrc	tempF,0		;3 check tempF.0
	rjmp	snde		;3
	cbi	PORTD,SND	;3 clear SND if tempF.0 = 0
	rjmp	sndd		;3
snde:	sbi	PORTD,SND	;3 set   SND if tempF.0 = 1
sndd:	brne	Sgen		;3
	sbi	PORTD,SND	;  set   SND for beeper power off
	ret

;***** Keyboard support:

;Scan local keyboard:
;Out: temp - scancode

SCAN:	ldi	temp,0xF7	;temp init
	ldi	tempD,8		;scan
sc1:	cbi	PORTB,CLK	;CLK <- 0
	rol	temp		;C <- temp.7..temp.0 <- C
	brcs	sc11
sc10:	cbi	PORTB,DATA	;DATA <- 0 or..
	rjmp	sc2
sc11:	sbi	PORTB,DATA	;DATA <- 1
sc2:	dec	tempD
	sbi	PORTB,CLK	;CLK <- 1
	sec
	sbic	PINB,RETL	;C <- RETL
	clc	
	brne	sc1
	rol	temp		;temp.0 <- C
	andi	temp,0x0F	;temp = scancode
	ret

;***** LCD support:

;Write nibble from temp to LCD:

LCD_WN:	andi	temp,0x0F	;mask unused bits
	ori	temp,0x10	;address=1
	rjmp	wa1

;Write address from temp to LCD:

LCD_WA:	andi	temp,0x0F	;mask unused bits
wa1:	lsl	temp
	lsl	temp
	lsl	temp
	ldi	tempD,5		;write 5 bits to LCD
w5:	cbi	PORTB,CLK	;CLK <- 0
	rol	temp
	brcs	w51
w50:	cbi	PORTB,DATA	;DATA <- 0 or..
	rjmp	w52
w51:	sbi	PORTB,DATA	;DATA <- 1
w52:	dec	tempD
	sbi	PORTB,CLK	;CLK <- 1
	brne	w5
	cbi	PORTB,LOAD	;LOAD <- 0
	sbi	PORTB,DATA
	sbi	PORTB,LOAD	;LOAD <- 1
	ret
		
;Font table:

FONT:	     ;FCBHADEG    FCBHADEG
	.DB 0b11101110, 0b01100000	;0, 1
	.DB 0b00101111, 0b01101101	;2, 3
	.DB 0b11100001, 0b11001101	;4, 5
	.DB 0b11001111, 0b01101000	;6, 7
	.DB 0b11101111, 0b11101101	;8, 9
	.DB 0b11101011, 0b01100111	;A, d
	.DB 0b10001111, 0b01000111	;E, o
	.DB 0b00000000, 0b00000100	;blank, _
	.DB 0b00001000,	0b11100011	;~, H
	.DB 0b10101011, 0b10000111	;P, t
	.DB 0b01000110,	0b11100101	;u, Y
	.DB 0b10000110, 0b10001010	;|_, |~

.equ	H	=4	;point

;Characters codes table:

.equ	BLANK	=0x0E	;character "blank" code
.equ	i_	=0x0E	;character "blank" code
.equ	iLL	=0x0F	;character "lower -" code
.equ	iHH	=0x10	;character "upper -" code
.equ	iHL	=0x16	;character "|_" code
.equ	iLH	=0x17	;character "|~" code
.equ	iA	=0x0A	;character "A" code
.equ	iD	=0x0B	;character "d" code
.equ	iE	=0x0C	;character "E" code
.equ	iH	=0x11	;character "H" code
.equ	iO	=0x0D	;character "o" code
.equ	iP	=0x12	;character "P" code
.equ	iS	=0x05	;character "S" code
.equ	iT	=0x13	;character "t" code
.equ	iU	=0x14	;character "u" code
.equ	iY	=0x15	;character "Y" code

;String table:

StrT:
	.DB iD,i_,iP,i_,iS,iH,iS,iY,iE,i_
	.DB iD,iS,iP,iS,i_,i_,i_,i_,iE,iS

;Shape string table:

ShpT:
	.DB iLL,iLH,iHL,iLH
	.DB iLL,iLH,iHL,iLL
	.DB iHH,iHL,iLH,iHH
	.DB iLL,iLL,iLL,iLL
	.DB iHH,iHH,iHH,iHH
	
;Sync string table:

SynT:
	.DB iA,iU,iT,iO
	.DB iLL,iLL,iLH,iHH
	.DB iHH,iHH,iHL,iLL
	
;Store string table:

SttT:
	.DB iE,iE,iP,i_
